@echo off
rem # ------------------------------------------------------------
rem # Company: Software- und Organisations-Service GmbH
rem # Author : Oliver Haufe <oliver.haufe@sos-berlin.com>
rem # ------------------------------------------------------------

SETLOCAL

set SCHEDULER_HTTP_PORT=%~1
set SCHEDULER_DATA=%~2
set SCHEDULER_LOG_DIR=%~3
set HTTPS_PORT_OPTION=%~4
set SERVICE_USER=%~5
set "SERVICE_PASSWORD=%~6"
set SCHEDULER_HOME=%~7
set SETUP_LOG_FILE=%~8
set /a RC=0

if not defined SCHEDULER_HTTP_PORT (
  echo Please enter the http port of the JobScheduler Agent
  goto usage
)
call :extract_port %SCHEDULER_HTTP_PORT%
if x%SERVICE_USER% == x\ set SERVICE_USER=
goto install

:usage
echo Usage: %~nx0 "http-port" "[agent-data-dir]" "[log-dir]" "[https-port]" "[service-user]" "[service-password]" "[agent-home-dir]"
call :more_info
echo.
pause
exit /b 0

:more_info
echo see also https://kb.sos-berlin.com/x/kQiX for more information.
goto final

:normalize_home
set SCHEDULER_HOME=%~dp1
set SCHEDULER_HOME=%SCHEDULER_HOME:~0,-1%
goto final

:extract_port
for /F "tokens=1,2 delims=:" %%i in ("%~1") do (
  if x%%j==x (
    call :set_port %%i
    goto final
  )
  call :set_port %%j
  goto final
)

:set_port
set SCHEDULER_PORT=%~1
goto final

:install_service 
if defined SERVICE_USER (
"%SERVICE_INSTALL%" //IS/%SERVICE_NAME% ^
  --DisplayName="%SERVICE_DISPLAY_NAME%" ^
  --Description="%SERVICE_DESCRIPTION%" ^
  --Install="%SERVICE_INSTALL%" ^
  --Startup="%SERVICE_STARTUP%" ^
  --LogPath="%SERVICE_LOGPATH%" ^
  --LogPrefix="%SERVICE_LOGPREFIX%" ^
  --LogLevel="%SERVICE_LOGLEVEL%" ^
  --StartImage="%SERVICE_STARTIMAGE%" ^
  --StartPath="%SCHEDULER_HOME%" ^
  --StartMode="%SERVICE_STARTMODE%" ^
  --StartParams="%SERVICE_STARTPARAMS%" ^
  --StopImage="%SERVICE_STOPIMAGE%" ^
  --StopPath="%SCHEDULER_HOME%" ^
  --StopMode="%SERVICE_STOPMODE%" ^
  --StopParams="%SERVICE_STOPPARAMS%" ^
  --ServiceUser="%SERVICE_USER%" ^
  --ServicePassword="%SERVICE_PASSWORD%"
) else (
"%SERVICE_INSTALL%" //IS/%SERVICE_NAME% ^
  --DisplayName="%SERVICE_DISPLAY_NAME%" ^
  --Description="%SERVICE_DESCRIPTION%" ^
  --Install="%SERVICE_INSTALL%" ^
  --Startup="%SERVICE_STARTUP%" ^
  --LogPath="%SERVICE_LOGPATH%" ^
  --LogPrefix="%SERVICE_LOGPREFIX%" ^
  --LogLevel="%SERVICE_LOGLEVEL%" ^
  --StartImage="%SERVICE_STARTIMAGE%" ^
  --StartPath="%SCHEDULER_HOME%" ^
  --StartMode="%SERVICE_STARTMODE%" ^
  --StartParams="%SERVICE_STARTPARAMS%" ^
  --StopImage="%SERVICE_STOPIMAGE%" ^
  --StopPath="%SCHEDULER_HOME%" ^
  --StopMode="%SERVICE_STOPMODE%" ^
  --StopParams="%SERVICE_STOPPARAMS%"
)
goto final

:update
if defined SERVICE_USER (
"%SERVICE_INSTALL%" //US/%SERVICE_NAME% ^
  --DisplayName="%SERVICE_DISPLAY_NAME%" ^
  --Description="%SERVICE_DESCRIPTION%" ^
  --Install="%SERVICE_INSTALL%" ^
  --LogPath="%SERVICE_LOGPATH%" ^
  --LogPrefix="%SERVICE_LOGPREFIX%" ^
  --StartImage="%SERVICE_STARTIMAGE%" ^
  --StartPath="%SCHEDULER_HOME%" ^
  --StartMode="%SERVICE_STARTMODE%" ^
  --StartParams="%SERVICE_STARTPARAMS%" ^
  --StopImage="%SERVICE_STOPIMAGE%" ^
  --StopPath="%SCHEDULER_HOME%" ^
  --StopMode="%SERVICE_STOPMODE%" ^
  --StopParams="%SERVICE_STOPPARAMS%" ^
  --ServiceUser="%SERVICE_USER%" ^
  --ServicePassword="%SERVICE_PASSWORD%"
) else (
"%SERVICE_INSTALL%" //US/%SERVICE_NAME% ^
  --DisplayName="%SERVICE_DISPLAY_NAME%" ^
  --Description="%SERVICE_DESCRIPTION%" ^
  --Install="%SERVICE_INSTALL%" ^
  --LogPath="%SERVICE_LOGPATH%" ^
  --LogPrefix="%SERVICE_LOGPREFIX%" ^
  --StartImage="%SERVICE_STARTIMAGE%" ^
  --StartPath="%SCHEDULER_HOME%" ^
  --StartMode="%SERVICE_STARTMODE%" ^
  --StartParams="%SERVICE_STARTPARAMS%" ^
  --StopImage="%SERVICE_STOPIMAGE%" ^
  --StopPath="%SCHEDULER_HOME%" ^
  --StopMode="%SERVICE_STOPMODE%" ^
  --StopParams="%SERVICE_STOPPARAMS%"
)
goto final

:install 
set SERVICE_NAME=sos_jobscheduler_agent_%SCHEDULER_PORT%
set SERVICE_DISPLAY_NAME=SOS JobScheduler Agent -port=%SCHEDULER_PORT%
set SERVICE_DESCRIPTION=JobScheduler Universal Agent

if not defined SCHEDULER_HOME set SCHEDULER_HOME=%~dp0..
call :normalize_home "%SCHEDULER_HOME%\dummy"
if not defined SCHEDULER_DATA set SCHEDULER_DATA="%SCHEDULER_HOME%\var_%SCHEDULER_PORT%"
if not defined SCHEDULER_LOG_DIR set SCHEDULER_LOG_DIR=%SCHEDULER_DATA%\logs
set SERVICE_STARTSCRIPT=bin\jobscheduler_agent_%SCHEDULER_PORT%.cmd
set SERVICE_INSTALL=%SCHEDULER_HOME%\service\%SERVICE_NAME%.exe
set SERVICE_MANAGER=%SCHEDULER_HOME%\service\%SERVICE_NAME%w.exe
set SERVICE_INSTALL_TEMPLATE=%SCHEDULER_HOME%\service\%PROCESSOR_ARCHITECTURE%\prunsrv.exe
set SERVICE_MANAGER_TEMPLATE=%SCHEDULER_HOME%\service\manager\prunmgr.exe
 
@REM Service Log Configuration
set SERVICE_LOGPREFIX=jobscheduler_agent_%SCHEDULER_PORT%-service
set SERVICE_LOGPATH=%SCHEDULER_LOG_DIR%
set SERVICE_LOGLEVEL=Info

@REM Startup Configuration
set SERVICE_STARTUP=auto
set SERVICE_STARTMODE=exe
set SERVICE_STARTIMAGE=%ComSpec%
set SERVICE_STARTPARAMS=/C;\"bin\jobscheduler_agent.cmd start -http-port=%SCHEDULER_HTTP_PORT% %HTTPS_PORT_OPTION% -data-directory=\\"%SCHEDULER_DATA%\\"\"
if exist "%SCHEDULER_HOME%\%SERVICE_STARTSCRIPT%" set SERVICE_STARTPARAMS=/C;\"%SERVICE_STARTSCRIPT% start\"
 
@REM Shutdown Configuration
set SERVICE_STOPMODE=exe
set SERVICE_STOPIMAGE=%ComSpec%
set SERVICE_STOPPARAMS=/C;\"bin\jobscheduler_agent.cmd stop -http-port=%SCHEDULER_HTTP_PORT%\"
if exist "%SCHEDULER_HOME%\%SERVICE_STARTSCRIPT%" set SERVICE_STOPPARAMS=/C;\"%SERVICE_STARTSCRIPT% stop\"

if not exist "%SERVICE_INSTALL%" (
  echo ...try to copy "%SERVICE_INSTALL_TEMPLATE%" to "%SERVICE_INSTALL%"
  copy /Y "%SERVICE_INSTALL_TEMPLATE%" "%SERVICE_INSTALL%"
  if ERRORLEVEL 1 (
    set /a RC=%ERRORLEVEL%
    goto finally
  )
)
if not exist "%SERVICE_MANAGER%" (
  echo ...try to copy "%SERVICE_MANAGER_TEMPLATE%" to "%SERVICE_MANAGER%"
  copy /Y "%SERVICE_MANAGER_TEMPLATE%" "%SERVICE_MANAGER%"
  if ERRORLEVEL 1 (
    set /a RC=%ERRORLEVEL%
    goto finally
  )
)

@REM check if Windows service is already installed
sc.exe query %SERVICE_NAME% >nul
if not %ERRORLEVEL% == 1060 (
	echo "%SERVICE_NAME%" service already installed.
	goto update_service
)

@REM install or update service
call :install_service
if ERRORLEVEL 1 (
  set /a RC=%ERRORLEVEL%
  echo Failed to install "%SERVICE_NAME%" service. Refer to log in "%SERVICE_LOGPATH%"
  goto finally
)
echo "%SERVICE_NAME%" service has been installed
goto install_update_service_end
:update_service
call :update
if ERRORLEVEL 1 (
  set /a RC=%ERRORLEVEL%
  echo Failed to update "%SERVICE_NAME%" service. Refer to log in "%SERVICE_LOGPATH%"
  goto finally
)
echo "%SERVICE_NAME%" service has been updated
:install_update_service_end
 
echo Please use "%SERVICE_MANAGER%" to change service configurations

@REM start service
@REM "%SERVICE_INSTALL%" //ES/%SERVICE_NAME%
@REM if ERRORLEVEL 1 (
@REM   set /a RC=%ERRORLEVEL%
@REM   echo Failed to start "%SERVICE_NAME%" service. Refer to log in "%SERVICE_LOGPATH%"
@REM   goto finally
@REM )
@REM echo "%SERVICE_NAME%" service has been started


:finally
if %RC% neq 0 (
  if defined SETUP_LOG_FILE (
    echo return_code = 1 > "%SERVICE_LOGPATH%\install_result.log"
    echo install_log = %SETUP_LOG_FILE% >> "%SERVICE_LOGPATH%\install_result.log"
    exit /b 0
  )
  exit /b %RC%
)
exit /b %ERRORLEVEL%

ENDLOCAL 
:final